@@ -22,6 +22,7 @@
 #include "job_p.h"
 #include "scheduler.h"
 #include "slave.h"
+#include "kprotocolinfo.h"
 #include <kdirnotify.h>
 #include <QTimer>
 #include <QDebug>
@@ -38,7 +39,7 @@ void SimpleJobPrivate::simpleJobInit()
 {
     Q_Q(SimpleJob);
     if (!m_url.isValid() || m_url.scheme().isEmpty()) {
-        qWarning() << "Invalid URL:" << m_url;
+        qCWarning(KIO_CORE) << "Invalid URL:" << m_url;
         q->setError(ERR_MALFORMED_URL);
         q->setErrorText(m_url.toString());
         QTimer::singleShot(0, q, SLOT(slotFinished()));
@@ -55,7 +56,7 @@ bool SimpleJob::doKill()
         d->m_extraFlags |= JobPrivate::EF_KillCalled;
         Scheduler::cancelJob(this); // deletes the slave if not 0
     } else {
-        qWarning() << this << "This is overkill.";
+        qCWarning(KIO_CORE) << this << "This is overkill.";
     }
     return Job::doKill();
 }
@@ -160,16 +161,16 @@ void SimpleJobPrivate::start(Slave *slave)
 
     const QVariant windowIdProp = q->property("window-id"); // see KJobWidgets::setWindow
     if (windowIdProp.isValid()) {
-        m_outgoingMetaData.insert("window-id", QString::number(windowIdProp.toULongLong()));
+        m_outgoingMetaData.insert(QStringLiteral("window-id"), QString::number(windowIdProp.toULongLong()));
     }
 
     const QVariant userTimestampProp = q->property("userTimestamp"); // see KJobWidgets::updateUserTimestamp
     if (userTimestampProp.isValid()) {
-        m_outgoingMetaData.insert("user-timestamp", QString::number(userTimestampProp.toULongLong()));
+        m_outgoingMetaData.insert(QStringLiteral("user-timestamp"), QString::number(userTimestampProp.toULongLong()));
     }
 
     if (q->uiDelegate() == 0) {            // not interactive
-        m_outgoingMetaData.insert("no-auth-prompt", "true");
+        m_outgoingMetaData.insert(QStringLiteral("no-auth-prompt"), QStringLiteral("true"));
     }
 
     if (!m_outgoingMetaData.isEmpty()) {
@@ -215,7 +216,8 @@ void SimpleJob::slotFinished()
                 QUrl src, dst;
                 QDataStream str(d->m_packedArgs);
                 str >> src >> dst;
-                if (src.adjusted(QUrl::RemoveFilename) == dst.adjusted(QUrl::RemoveFilename)) { // For the user, moving isn't renaming. Only renaming is.
+                if (src.adjusted(QUrl::RemoveFilename) == dst.adjusted(QUrl::RemoveFilename) // For the user, moving isn't renaming. Only renaming is.
+                    && !KProtocolInfo::slaveHandlesNotify(dst.scheme()).contains(QLatin1String("Rename"))) {
                     org::kde::KDirNotify::emitFileRenamed(src, dst);
                 }
 
@@ -302,7 +304,7 @@ int SimpleJobPrivate::requestMessageBox(int _type, const QString &text, const QS
         return m_uiDelegateExtension->requestMessageBox(type, text, caption, buttonYes, buttonNo,
                 iconYes, iconNo, dontAskAgainName, sslMetaData);
     }
-    qWarning() << "JobUiDelegate not set! Returing -1";
+    qCWarning(KIO_CORE) << "JobUiDelegate not set! Returing -1";
     return -1;
 }
 
@@ -384,7 +386,7 @@ SimpleJob *KIO::mount(bool ro, const QByteArray &fstype, const QString &dev, con
 {
     KIO_ARGS << int(1) << qint8(ro ? 1 : 0)
              << QString::fromLatin1(fstype) << dev << point;
-    SimpleJob *job = special(QUrl("file:///"), packedArgs, flags);
+    SimpleJob *job = special(QUrl(QStringLiteral("file:///")), packedArgs, flags);
     if (!(flags & HideProgressInfo)) {
         KIO::JobPrivate::emitMounting(job, dev, point);
     }
@@ -394,7 +396,7 @@ SimpleJob *KIO::mount(bool ro, const QByteArray &fstype, const QString &dev, con
 SimpleJob *KIO::unmount(const QString &point, JobFlags flags)
 {
     KIO_ARGS << int(2) << point;
-    SimpleJob *job = special(QUrl("file:///"), packedArgs, flags);
+    SimpleJob *job = special(QUrl(QStringLiteral("file:///")), packedArgs, flags);
     if (!(flags & HideProgressInfo)) {
         KIO::JobPrivate::emitUnmounting(job, point);
     }
@@ -405,7 +407,7 @@ SimpleJob *KIO::unmount(const QString &point, JobFlags flags)
 
 SimpleJob *KIO::http_update_cache(const QUrl &url, bool no_cache, const QDateTime &expireDate)
 {
-    Q_ASSERT(url.scheme() == "http" || url.scheme() == "https");
+    Q_ASSERT(url.scheme() == QLatin1String("http") || url.scheme() == QLatin1String("https"));
     // Send http update_cache command (2)
     KIO_ARGS << (int)2 << url << no_cache << qlonglong(expireDate.toMSecsSinceEpoch() / 1000);
     SimpleJob *job = SimpleJobPrivate::newJob(url, CMD_SPECIAL, packedArgs);
