@@ -46,7 +46,7 @@ void TransferJob::slotData(const QByteArray &_data)
 {
     Q_D(TransferJob);
     if (d->m_command == CMD_GET && !d->m_isMimetypeEmitted) {
-        qWarning() << "mimeType() not emitted when sending first data!; job URL ="
+        qCWarning(KIO_CORE) << "mimeType() not emitted when sending first data!; job URL ="
                    << d->m_url << "data size =" << _data.size();
     }
     // shut up the warning, HACK: downside is that it changes the meaning of the variable
@@ -67,8 +67,8 @@ void TransferJob::slotRedirection(const QUrl &url)
 {
     Q_D(TransferJob);
     //qDebug() << url;
-    if (!KUrlAuthorized::authorizeUrlAction("redirect", d->m_url, url)) {
-        qWarning() << "Redirection from" << d->m_url << "to" << url << "REJECTED!";
+    if (!KUrlAuthorized::authorizeUrlAction(QStringLiteral("redirect"), d->m_url, url)) {
+        qCWarning(KIO_CORE) << "Redirection from" << d->m_url << "to" << url << "REJECTED!";
         return;
     }
 
@@ -82,7 +82,12 @@ void TransferJob::slotRedirection(const QUrl &url)
     } else {
         d->m_redirectionURL = url; // We'll remember that when the job finishes
         d->m_redirectionList.append(url);
-        d->m_outgoingMetaData["ssl_was_in_use"] = d->m_incomingMetaData["ssl_in_use"];
+        QString sslInUse = queryMetaData(QStringLiteral("ssl_in_use"));
+        if (!sslInUse.isNull()) { // the key is present
+            addMetaData(QStringLiteral("ssl_was_in_use"), sslInUse);
+        } else {
+            addMetaData(QStringLiteral("ssl_was_in_use"), QStringLiteral("FALSE"));
+        }
         // Tell the user that we haven't finished yet
         emit redirection(this, d->m_redirectionURL);
     }
@@ -96,10 +101,16 @@ void TransferJob::slotFinished()
     if (!d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid()) {
 
         //qDebug() << "Redirection to" << m_redirectionURL;
-        if (queryMetaData("permanent-redirect") == "true") {
+        if (queryMetaData(QStringLiteral("permanent-redirect")) == QLatin1String("true")) {
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
         }
 
+        if (queryMetaData(QStringLiteral("redirect-to-get")) == QLatin1String("true")) {
+            d->m_command = CMD_GET;
+            d->m_outgoingMetaData.remove(QStringLiteral("CustomHTTPMethod"));
+            d->m_outgoingMetaData.remove(QStringLiteral("content-type"));
+        }
+
         if (d->m_redirectionHandlingEnabled) {
             // Honour the redirection
             // We take the approach of "redirecting this same job"
@@ -107,8 +118,8 @@ void TransferJob::slotFinished()
             // happens (unpacking+repacking)
             d->staticData.truncate(0);
             d->m_incomingMetaData.clear();
-            if (queryMetaData("cache") != "reload") {
-                addMetaData("cache", "refresh");
+            if (queryMetaData(QStringLiteral("cache")) != QLatin1String("reload")) {
+                addMetaData(QStringLiteral("cache"), QStringLiteral("refresh"));
             }
             d->m_internalSuspended = false;
             // The very tricky part is the packed arguments business
@@ -116,7 +127,9 @@ void TransferJob::slotFinished()
             QUrl dummyUrl;
             QDataStream istream(d->m_packedArgs);
             switch (d->m_command) {
-            case CMD_GET: {
+            case CMD_GET:
+            case CMD_STAT:
+            case CMD_DEL: {
                 d->m_packedArgs.truncate(0);
                 QDataStream stream(&d->m_packedArgs, QIODevice::WriteOnly);
                 stream << d->m_redirectionURL;
@@ -135,8 +148,8 @@ void TransferJob::slotFinished()
                 int specialcmd;
                 istream >> specialcmd;
                 if (specialcmd == 1) { // HTTP POST
-                    d->m_outgoingMetaData.remove(QLatin1String("content-type"));
-                    addMetaData("cache", "reload");
+                    d->m_outgoingMetaData.remove(QStringLiteral("content-type"));
+                    addMetaData(QStringLiteral("cache"), QStringLiteral("reload"));
                     d->m_packedArgs.truncate(0);
                     QDataStream stream(&d->m_packedArgs, QIODevice::WriteOnly);
                     stream << d->m_redirectionURL;
@@ -177,7 +190,7 @@ void TransferJob::sendAsyncData(const QByteArray &dataForSlave)
     d->m_extraFlags &= ~JobPrivate::EF_TransferJobNeedData;
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 void TransferJob::setReportDataSent(bool enabled)
 {
     Q_D(TransferJob);
@@ -189,7 +202,7 @@ void TransferJob::setReportDataSent(bool enabled)
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 bool TransferJob::reportDataSent() const
 {
     return (d_func()->m_extraFlags & JobPrivate::EF_TransferJobDataSent);
@@ -246,7 +259,7 @@ void TransferJob::slotMimetype(const QString &type)
     Q_D(TransferJob);
     d->m_mimetype = type;
     if (d->m_command == CMD_GET && d->m_isMimetypeEmitted) {
-        qWarning() << "mimetype() emitted again, or after sending first data!; job URL =" << d->m_url;
+        qCWarning(KIO_CORE) << "mimetype() emitted again, or after sending first data!; job URL =" << d->m_url;
     }
     d->m_isMimetypeEmitted = true;
     emit mimetype(this, type);
@@ -293,10 +306,20 @@ void TransferJobPrivate::start(Slave *slave)
     q->connect(slave, SIGNAL(data(QByteArray)),
                SLOT(slotData(QByteArray)));
 
-    if (m_outgoingDataSource)
-        q->connect(slave, SIGNAL(dataReq()),
-                   SLOT(slotDataReqFromDevice()));
-    else
+    if (m_outgoingDataSource) {
+        if (m_extraFlags & JobPrivate::EF_TransferJobAsync) {
+            q->connect(m_outgoingDataSource, SIGNAL(readyRead()),
+                    SLOT(slotDataReqFromDevice()));
+            q->connect(m_outgoingDataSource, SIGNAL(readChannelFinished()),
+                    SLOT(slotIODeviceClosed()));
+            if (m_outgoingDataSource->bytesAvailable() > 0) {
+                QMetaObject::invokeMethod(q, "slotDataReqFromDevice", Qt::QueuedConnection);
+            }
+        } else {
+            q->connect(slave, SIGNAL(dataReq()),
+                    SLOT(slotDataReqFromDevice()));
+        }
+    } else
         q->connect(slave, SIGNAL(dataReq()),
                    SLOT(slotDataReq()));
 
@@ -316,7 +339,7 @@ void TransferJobPrivate::start(Slave *slave)
                SLOT(slotCanResume(KIO::filesize_t)));
 
     if (slave->suspended()) {
-        m_mimetype = "unknown";
+        m_mimetype = QStringLiteral("unknown");
         // WABA: The slave was put on hold. Resume operation.
         slave->resume();
     }
@@ -368,17 +391,31 @@ void TransferJobPrivate::slotDataReqFromDevice()
 {
     Q_Q(TransferJob);
 
+    bool done = false;
     QByteArray dataForSlave;
 
     m_extraFlags |= JobPrivate::EF_TransferJobNeedData;
 
     if (m_outgoingDataSource) {
-        dataForSlave = m_outgoingDataSource.data()->read(MAX_READ_BUF_SIZE);
+        dataForSlave.resize(MAX_READ_BUF_SIZE);
+
+        //Code inspired in QNonContiguousByteDevice
+        qint64 bytesRead = m_outgoingDataSource->read(dataForSlave.data(), MAX_READ_BUF_SIZE);
+        if (bytesRead >= 0) {
+            dataForSlave.resize(bytesRead);
+        } else {
+            dataForSlave.clear();
+        }
+        done = ((bytesRead == -1) || (bytesRead == 0 && m_outgoingDataSource->atEnd() && !m_outgoingDataSource->isSequential()));
+
+        if (!done) {
+            QMetaObject::invokeMethod(q, "slotDataReqFromDevice", Qt::QueuedConnection);
+        }
     }
 
     if (dataForSlave.isEmpty()) {
         emit q->dataReq(q, dataForSlave);
-        if (m_extraFlags & JobPrivate::EF_TransferJobAsync) {
+        if (!done && (m_extraFlags & JobPrivate::EF_TransferJobAsync)) {
             return;
         }
     }
@@ -392,6 +429,27 @@ void TransferJobPrivate::slotDataReqFromDevice()
     }
 }
 
+void TransferJobPrivate::slotIODeviceClosed()
+{
+    Q_Q(TransferJob);
+    const QByteArray remainder = m_outgoingDataSource->readAll();
+    if (!remainder.isEmpty()) {
+        m_extraFlags |= JobPrivate::EF_TransferJobNeedData;
+        q->sendAsyncData(remainder);
+    }
+
+    m_extraFlags |= JobPrivate::EF_TransferJobNeedData;
+
+    //We send an empty data array to indicate the stream is over
+    q->sendAsyncData(QByteArray());
+
+    if (m_subJob) {
+        // Bitburger protocol in action
+        internalSuspend(); // Wait for more data from subJob.
+        m_subJob->d_func()->internalResume(); // Ask for more!
+    }
+}
+
 void TransferJob::slotResult(KJob *job)
 {
     Q_D(TransferJob);
@@ -408,7 +466,7 @@ void TransferJob::slotResult(KJob *job)
 
 void TransferJob::setModificationTime(const QDateTime &mtime)
 {
-    addMetaData("modified", mtime.toString(Qt::ISODate));
+    addMetaData(QStringLiteral("modified"), mtime.toString(Qt::ISODate));
 }
 
 TransferJob *KIO::get(const QUrl &url, LoadType reload, JobFlags flags)
@@ -418,9 +476,9 @@ TransferJob *KIO::get(const QUrl &url, LoadType reload, JobFlags flags)
     TransferJob *job = TransferJobPrivate::newJob(url, CMD_GET, packedArgs,
                        QByteArray(), flags);
     if (reload == Reload) {
-        job->addMetaData("cache", "reload");
+        job->addMetaData(QStringLiteral("cache"), QStringLiteral("reload"));
     }
     return job;
 }
 
-#include "moc_transferjob.cpp"
\ No newline at end of file
+#include "moc_transferjob.cpp"
