@@ -77,6 +77,15 @@
 #include <kdirnotify.h>
 #include <ioslave_defaults.h>
 
+Q_LOGGING_CATEGORY(KIO_FILE, "kf5.kio.kio_file")
+
+// Pseudo plugin class to embed meta data
+class KIOPluginForMetaData : public QObject
+{
+    Q_OBJECT
+    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.file" FILE "file.json")
+};
+
 using namespace KIO;
 
 #define MAX_IPC_SIZE (1024*32)
@@ -90,7 +99,7 @@ static void appendACLAtoms(const QByteArray &path, UDSEntry &entry,
 extern "C" Q_DECL_EXPORT int kdemain(int argc, char **argv)
 {
     QCoreApplication app(argc, argv);   // needed for QSocketNotifier
-    app.setApplicationName(QLatin1String("kio_file"));
+    app.setApplicationName(QStringLiteral("kio_file"));
 
     if (argc != 4) {
         fprintf(stderr, "Usage: kio_file protocol domain-socket1 domain-socket2\n");
@@ -145,7 +154,7 @@ static QFile::Permissions modeToQFilePermissions(int mode)
 }
 
 FileProtocol::FileProtocol(const QByteArray &pool, const QByteArray &app)
-    : SlaveBase("file", pool, app), mFile(0)
+    : SlaveBase(QByteArrayLiteral("file"), pool, app), mFile(0)
 {
 }
 
@@ -169,8 +178,8 @@ int FileProtocol::setACL(const char *path, mode_t perm, bool directoryDefault)
     int ret = 0;
 #if HAVE_POSIX_ACL
 
-    const QString ACLString = metaData(QLatin1String("ACL_STRING"));
-    const QString defaultACLString = metaData(QLatin1String("DEFAULT_ACL_STRING"));
+    const QString ACLString = metaData(QStringLiteral("ACL_STRING"));
+    const QString defaultACLString = metaData(QStringLiteral("DEFAULT_ACL_STRING"));
     // Empty strings mean leave as is
     if (!ACLString.isEmpty()) {
         acl_t acl = 0;
@@ -268,7 +277,7 @@ void FileProtocol::mkdir(const QUrl &url, int permissions)
     // qDebug() << path << "permission=" << permissions;
 
     // Remove existing file or symlink, if requested (#151851)
-    if (metaData(QLatin1String("overwrite")) == QLatin1String("true")) {
+    if (metaData(QStringLiteral("overwrite")) == QLatin1String("true")) {
         QFile::remove(path);
     }
 
@@ -334,6 +343,7 @@ void FileProtocol::get(const QUrl &url)
     }
 
 #if HAVE_FADVISE
+    //TODO check return code
     posix_fadvise(f.handle(), 0, 0, POSIX_FADV_SEQUENTIAL);
 #endif
 
@@ -350,9 +360,9 @@ void FileProtocol::get(const QUrl &url)
 
     KIO::filesize_t processed_size = 0;
 
-    QString resumeOffset = metaData(QLatin1String("range-start"));
+    QString resumeOffset = metaData(QStringLiteral("range-start"));
     if (resumeOffset.isEmpty()) {
-        resumeOffset = metaData(QLatin1String("resume")); // old name
+        resumeOffset = metaData(QStringLiteral("resume")); // old name
     }
     if (!resumeOffset.isEmpty()) {
         bool ok;
@@ -488,7 +498,7 @@ void FileProtocol::write(const QByteArray &data)
             error(KIO::ERR_DISK_FULL, mFile->fileName());
             close();
         } else {
-            qWarning() << "Couldn't write. Error:" << mFile->errorString();
+            qCWarning(KIO_FILE) << "Couldn't write. Error:" << mFile->errorString();
             error(KIO::ERR_CANNOT_WRITE, mFile->fileName());
             close();
         }
@@ -530,7 +540,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
     QString dest_part(dest_orig + QLatin1String(".part"));
 
     QT_STATBUF buff_orig;
-    const bool bOrigExists = (QT_LSTAT(QFile::encodeName(dest_part).constData(), &buff_orig) != -1);
+    const bool bOrigExists = (QT_LSTAT(QFile::encodeName(dest_orig).constData(), &buff_orig) != -1);
     bool bPartExists = false;
     const bool bMarkPartial = config()->readEntry("MarkPartial", true);
 
@@ -594,17 +604,13 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                 if ((_flags & KIO::Resume)) {
                     f.open(QIODevice::ReadWrite | QIODevice::Append);
                 } else {
-                    // WABA: Make sure that we keep writing permissions ourselves,
-                    // otherwise we can be in for a surprise on NFS.
-                    mode_t initialMode;
-                    if (_mode != -1) {
-                        initialMode = _mode | S_IWUSR | S_IRUSR;
-                    } else {
-                        initialMode = 0666;
-                    }
-
                     f.open(QIODevice::Truncate | QIODevice::WriteOnly);
-                    f.setPermissions(modeToQFilePermissions(initialMode));
+                    if (_mode != -1) {
+                        // WABA: Make sure that we keep writing permissions ourselves,
+                        // otherwise we can be in for a surprise on NFS.
+                        mode_t initialMode = _mode | S_IWUSR | S_IRUSR;
+                        f.setPermissions(modeToQFilePermissions(initialMode));
+                    }
                 }
 
                 if (!f.isOpen()) {
@@ -625,7 +631,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                     error(KIO::ERR_DISK_FULL, dest_orig);
                     result = -2; // means: remove dest file
                 } else {
-                    qWarning() << "Couldn't write. Error:" << f.errorString();
+                    qCWarning(KIO_FILE) << "Couldn't write. Error:" << f.errorString();
                     error(KIO::ERR_CANNOT_WRITE, dest_orig);
                     result = -1;
                 }
@@ -660,7 +666,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
     f.close();
 
     if (f.error() != QFile::NoError) {
-        qWarning() << "Error when closing file descriptor:" << f.errorString();
+        qCWarning(KIO_FILE) << "Error when closing file descriptor:" << f.errorString();
         error(KIO::ERR_CANNOT_WRITE, dest_orig);
         return;
     }
@@ -674,7 +680,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
         }
 
         if (!QFile::rename(dest, dest_orig)) {
-            qWarning() << " Couldn't rename " << dest << " to " << dest_orig;
+            qCWarning(KIO_FILE) << " Couldn't rename " << dest << " to " << dest_orig;
             error(KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig);
             return;
         }
@@ -693,7 +699,7 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
     }
 
     // set modification time
-    const QString mtimeStr = metaData(QLatin1String("modified"));
+    const QString mtimeStr = metaData(QStringLiteral("modified"));
     if (!mtimeStr.isEmpty()) {
         QDateTime dt = QDateTime::fromString(mtimeStr, Qt::ISODate);
         if (dt.isValid()) {
@@ -778,8 +784,29 @@ bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &pat
 
         if ((buff.st_mode & QT_STAT_MASK) == QT_STAT_LNK) {
 
+#ifdef Q_OS_WIN
             const QString linkTarget = QFile::symLinkTarget(QFile::decodeName(path));
-
+#else
+            // Use readlink on Unix because symLinkTarget turns relative targets into absolute (#352927)
+            const off_t lowerLimit = 1;
+            const off_t upperLimit = 1024;
+            size_t bufferSize = qBound(lowerLimit, buff.st_size, upperLimit);
+            QByteArray linkTargetBuffer;
+            linkTargetBuffer.resize(bufferSize);
+            while (true) {
+                ssize_t n = readlink(path.constData(), linkTargetBuffer.data(), bufferSize);
+                if (n < 0 && errno != ERANGE) {
+                    qCWarning(KIO_FILE) << "readlink failed!" << path;
+                    return false;
+                } else if (n > 0 && static_cast<size_t>(n) != bufferSize) {
+                    linkTargetBuffer.truncate(n);
+                    break;
+                }
+                bufferSize *= 2;
+                linkTargetBuffer.resize(bufferSize);
+            }
+            const QString linkTarget = QFile::decodeName(linkTargetBuffer);
+#endif
             entry.insert(KIO::UDSEntry::UDS_LINK_DEST, linkTarget);
 
             // A symlink -> follow it only if details>1
@@ -796,7 +823,7 @@ bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &pat
             }
         }
     } else {
-        // qWarning() << "lstat didn't work on " << path.data();
+        // qCWarning(KIO_FILE) << "lstat didn't work on " << path.data();
         return false;
     }
 
@@ -880,7 +907,7 @@ void FileProtocol::special(const QByteArray &data)
 
 static QStringList fallbackSystemPath()
 {
-    return QStringList() << QLatin1String("/sbin") << QLatin1String("/bin");
+    return QStringList() << QStringLiteral("/sbin") << QStringLiteral("/bin");
 }
 
 void FileProtocol::mount(bool _ro, const char *_fstype, const QString &_dev, const QString &_point)
@@ -936,9 +963,9 @@ void FileProtocol::mount(bool _ro, const char *_fstype, const QString &_dev, con
     bool fstype_empty = !_fstype || !*_fstype;
     QByteArray fstype = KShell::quoteArg(QString::fromLatin1(_fstype)).toLatin1(); // good guess
     QByteArray readonly = _ro ? "-r" : "";
-    QByteArray mountProg = QStandardPaths::findExecutable(QLatin1String("mount")).toLocal8Bit();
+    QByteArray mountProg = QStandardPaths::findExecutable(QStringLiteral("mount")).toLocal8Bit();
     if (mountProg.isEmpty()) {
-        mountProg = QStandardPaths::findExecutable(QLatin1String("mount"), fallbackSystemPath()).toLocal8Bit();
+        mountProg = QStandardPaths::findExecutable(QStringLiteral("mount"), fallbackSystemPath()).toLocal8Bit();
     }
     if (mountProg.isEmpty()) {
         error(KIO::ERR_CANNOT_MOUNT, i18n("Could not find program \"mount\""));
@@ -1110,9 +1137,9 @@ void FileProtocol::unmount(const QString &_point)
         return;
     }
 #else
-    QByteArray umountProg = QStandardPaths::findExecutable(QLatin1String("umount")).toLocal8Bit();
+    QByteArray umountProg = QStandardPaths::findExecutable(QStringLiteral("umount")).toLocal8Bit();
     if (umountProg.isEmpty()) {
-        umountProg = QStandardPaths::findExecutable(QLatin1String("umount"), fallbackSystemPath()).toLocal8Bit();
+        umountProg = QStandardPaths::findExecutable(QStringLiteral("umount"), fallbackSystemPath()).toLocal8Bit();
     }
     if (umountProg.isEmpty()) {
         error(KIO::ERR_CANNOT_UNMOUNT, i18n("Could not find program \"umount\""));
@@ -1144,9 +1171,9 @@ void FileProtocol::unmount(const QString &_point)
 bool FileProtocol::pmount(const QString &dev)
 {
 #ifndef _WIN32_WCE
-    QString pmountProg = QStandardPaths::findExecutable(QLatin1String("pmount"));
+    QString pmountProg = QStandardPaths::findExecutable(QStringLiteral("pmount"));
     if (pmountProg.isEmpty()) {
-        pmountProg = QStandardPaths::findExecutable(QLatin1String("pmount"), fallbackSystemPath());
+        pmountProg = QStandardPaths::findExecutable(QStringLiteral("pmount"), fallbackSystemPath());
     }
     if (pmountProg.isEmpty()) {
         return false;
@@ -1175,9 +1202,9 @@ bool FileProtocol::pumount(const QString &point)
         return false;
     }
 
-    QString pumountProg = QStandardPaths::findExecutable(QLatin1String("pumount"));
+    QString pumountProg = QStandardPaths::findExecutable(QStringLiteral("pumount"));
     if (pumountProg.isEmpty()) {
-        pumountProg = QStandardPaths::findExecutable(QLatin1String("pumount"), fallbackSystemPath());
+        pumountProg = QStandardPaths::findExecutable(QStringLiteral("pumount"), fallbackSystemPath());
     }
     if (pumountProg.isEmpty()) {
         return false;
@@ -1306,8 +1333,8 @@ void FileProtocol::fileSystemFreeSpace(const QUrl &url)
     if (url.isLocalFile()) {
         const KDiskFreeSpaceInfo spaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(url.toLocalFile());
         if (spaceInfo.isValid()) {
-            setMetaData(QString::fromLatin1("total"), QString::number(spaceInfo.size()));
-            setMetaData(QString::fromLatin1("available"), QString::number(spaceInfo.available()));
+            setMetaData(QStringLiteral("total"), QString::number(spaceInfo.size()));
+            setMetaData(QStringLiteral("available"), QString::number(spaceInfo.available()));
 
             finished();
         } else {
@@ -1330,3 +1357,6 @@ void FileProtocol::virtual_hook(int id, void *data)
     } break;
     }
 }
+
+// needed for JSON file embedding
+#include "file.moc"
