@@ -22,6 +22,7 @@
 
 #include "jobuidelegate.h"
 #include <kio/jobuidelegatefactory.h>
+#include "kio_widgets_debug.h"
 
 #include <kconfiggroup.h>
 #include <kjob.h>
@@ -98,11 +99,11 @@ public:
             m_windowList.insert(obj, windowId);
             connect(window, SIGNAL(destroyed(QObject*)),
                     this, SLOT(slotUnregisterWindow(QObject*)));
-            QDBusInterface("org.kde.kded5", "/kded", "org.kde.kded5").
-            call(QDBus::NoBlock, "registerWindowId", qlonglong(windowId));
+            QDBusInterface(QStringLiteral("org.kde.kded5"), QStringLiteral("/kded"), QStringLiteral("org.kde.kded5")).
+            call(QDBus::NoBlock, QStringLiteral("registerWindowId"), qlonglong(windowId));
         }
     }
-private Q_SLOTS:
+public Q_SLOTS:
     void slotUnregisterWindow(QObject *obj)
     {
         if (!obj) {
@@ -117,14 +118,14 @@ private Q_SLOTS:
         disconnect(it.key(), SIGNAL(destroyed(QObject*)),
                    this, SLOT(slotUnregisterWindow(QObject*)));
         m_windowList.erase(it);
-        QDBusInterface("org.kde.kded5", "/kded", "org.kde.kded5").
-        call(QDBus::NoBlock, "unregisterWindowId", qlonglong(windowId));
+        QDBusInterface(QStringLiteral("org.kde.kded5"), QStringLiteral("/kded"), QStringLiteral("org.kde.kded5")).
+        call(QDBus::NoBlock, QStringLiteral("unregisterWindowId"), qlonglong(windowId));
     }
 private:
     QMap<QObject *, WId> m_windowList;
 };
 
-Q_GLOBAL_STATIC(JobUiDelegateStatic, s_static);
+Q_GLOBAL_STATIC(JobUiDelegateStatic, s_static)
 
 void KIO::JobUiDelegate::setWindow(QWidget *window)
 {
@@ -132,6 +133,11 @@ void KIO::JobUiDelegate::setWindow(QWidget *window)
     s_static()->registerWindow(window);
 }
 
+void KIO::JobUiDelegate::unregisterWindow(QWidget *window)
+{
+    s_static()->slotUnregisterWindow(window);
+}
+
 KIO::RenameDialog_Result KIO::JobUiDelegate::askFileRename(KJob *job,
         const QString &caption,
         const QUrl &src,
@@ -180,17 +186,17 @@ bool KIO::JobUiDelegate::askDeleteConfirmation(const QList<QUrl> &urls,
     QString keyName;
     bool ask = (confirmationType == ForceConfirmation);
     if (!ask) {
-        KSharedConfigPtr kioConfig = KSharedConfig::openConfig("kiorc", KConfig::NoGlobals);
+        KSharedConfigPtr kioConfig = KSharedConfig::openConfig(QStringLiteral("kiorc"), KConfig::NoGlobals);
 
         switch (deletionType) {
         case Delete:
-            keyName = "ConfirmDelete";
+            keyName = QStringLiteral("ConfirmDelete");
             break;
         case Trash:
-            keyName = "ConfirmTrash";
+            keyName = QStringLiteral("ConfirmTrash");
             break;
         case EmptyTrash:
-            keyName = "ConfirmEmptyTrash";
+            keyName = QStringLiteral("ConfirmEmptyTrash");
             break;
         }
 
@@ -202,14 +208,14 @@ bool KIO::JobUiDelegate::askDeleteConfirmation(const QList<QUrl> &urls,
     if (ask) {
         QStringList prettyList;
         Q_FOREACH (const QUrl &url, urls) {
-            if (url.scheme() == "trash") {
+            if (url.scheme() == QLatin1String("trash")) {
                 QString path = url.path();
                 // HACK (#98983): remove "0-foo". Note that it works better than
                 // displaying KFileItem::name(), for files under a subdir.
-                path.remove(QRegExp("^/[0-9]*-"));
+                path.remove(QRegExp(QStringLiteral("^/[0-9]*-")));
                 prettyList.append(path);
             } else {
-                prettyList.append(url.toDisplayString());
+                prettyList.append(url.toDisplayString(QUrl::PreferLocalFile));
             }
         }
 
@@ -233,7 +239,7 @@ bool KIO::JobUiDelegate::askDeleteConfirmation(const QList<QUrl> &urls,
                          i18nc("@info", "Do you want to permanently delete all items from Trash? This action cannot be undone."),
                          QString(),
                          KGuiItem(i18nc("@action:button", "Empty Trash"),
-                                  QIcon::fromTheme("user-trash")),
+                                  QIcon::fromTheme(QStringLiteral("user-trash"))),
                          KStandardGuiItem::cancel(),
                          keyName, options);
             break;
@@ -244,7 +250,7 @@ bool KIO::JobUiDelegate::askDeleteConfirmation(const QList<QUrl> &urls,
                          i18np("Do you really want to move this item to the trash?", "Do you really want to move these %1 items to the trash?", prettyList.count()),
                          prettyList,
                          i18n("Move to Trash"),
-                         KGuiItem(i18nc("Verb", "&Trash"), "user-trash"),
+                         KGuiItem(i18nc("Verb", "&Trash"), QStringLiteral("user-trash")),
                          KStandardGuiItem::cancel(),
                          keyName, options);
         }
@@ -256,7 +262,7 @@ bool KIO::JobUiDelegate::askDeleteConfirmation(const QList<QUrl> &urls,
                 notificationGroup.writeEntry(keyName, true);
                 notificationGroup.sync();
 
-                KSharedConfigPtr kioConfig = KSharedConfig::openConfig("kiorc", KConfig::NoGlobals);
+                KSharedConfigPtr kioConfig = KSharedConfig::openConfig(QStringLiteral("kiorc"), KConfig::NoGlobals);
                 kioConfig->group("Confirmations").writeEntry(keyName, false);
             }
         }
@@ -276,7 +282,7 @@ int KIO::JobUiDelegate::requestMessageBox(KIO::JobUiDelegate::MessageBoxType typ
 
     //qDebug() << type << text << "caption=" << caption;
 
-    KConfig config("kioslaverc");
+    KConfig config(QStringLiteral("kioslaverc"));
     KMessageBox::setDontShowAgainConfig(&config);
 
     const KGuiItem buttonYesGui(buttonYes, iconYes);
@@ -312,7 +318,7 @@ int KIO::JobUiDelegate::requestMessageBox(KIO::JobUiDelegate::MessageBoxType typ
     case SSLMessageBox: {
         QPointer<KSslInfoDialog> kid(new KSslInfoDialog(window()));
         //### this is boilerplate code and appears in khtml_part.cpp almost unchanged!
-        const QStringList sl = sslMetaData.value(QLatin1String("ssl_peer_chain")).split('\x01', QString::SkipEmptyParts);
+        const QStringList sl = sslMetaData.value(QStringLiteral("ssl_peer_chain")).split('\x01', QString::SkipEmptyParts);
         QList<QSslCertificate> certChain;
         bool decodedOk = true;
         foreach (const QString &s, sl) {
@@ -326,13 +332,13 @@ int KIO::JobUiDelegate::requestMessageBox(KIO::JobUiDelegate::MessageBoxType typ
         if (decodedOk) {
             result = 1; // whatever
             kid->setSslInfo(certChain,
-                            sslMetaData.value(QLatin1String("ssl_peer_ip")),
+                            sslMetaData.value(QStringLiteral("ssl_peer_ip")),
                             text, // the URL
-                            sslMetaData.value(QLatin1String("ssl_protocol_version")),
-                            sslMetaData.value(QLatin1String("ssl_cipher")),
-                            sslMetaData.value(QLatin1String("ssl_cipher_used_bits")).toInt(),
-                            sslMetaData.value(QLatin1String("ssl_cipher_bits")).toInt(),
-                            KSslInfoDialog::errorsFromString(sslMetaData.value(QLatin1String("ssl_cert_errors"))));
+                            sslMetaData.value(QStringLiteral("ssl_protocol_version")),
+                            sslMetaData.value(QStringLiteral("ssl_cipher")),
+                            sslMetaData.value(QStringLiteral("ssl_cipher_used_bits")).toInt(),
+                            sslMetaData.value(QStringLiteral("ssl_cipher_bits")).toInt(),
+                            KSslInfoDialog::errorsFromString(sslMetaData.value(QStringLiteral("ssl_cert_errors"))));
             kid->exec();
         } else {
             result = -1;
@@ -345,7 +351,7 @@ int KIO::JobUiDelegate::requestMessageBox(KIO::JobUiDelegate::MessageBoxType typ
         break;
     }
     default:
-        qWarning() << "Unknown type" << type;
+        qCWarning(KIO_WIDGETS) << "Unknown type" << type;
         result = 0;
         break;
     }
@@ -355,15 +361,17 @@ int KIO::JobUiDelegate::requestMessageBox(KIO::JobUiDelegate::MessageBoxType typ
 
 KIO::ClipboardUpdater *KIO::JobUiDelegate::createClipboardUpdater(Job *job, ClipboardUpdaterMode mode)
 {
-    if (qobject_cast<QGuiApplication *>(qApp) != NULL) {
+    if (qobject_cast<QGuiApplication *>(qApp)) {
         return new KIO::ClipboardUpdater(job, mode);
     }
-    return NULL;
+    return nullptr;
 }
 
 void KIO::JobUiDelegate::updateUrlInClipboard(const QUrl &src, const QUrl &dest)
 {
-    KIO::ClipboardUpdater::update(src, dest);
+    if (qobject_cast<QGuiApplication *>(qApp)) {
+        KIO::ClipboardUpdater::update(src, dest);
+    }
 }
 
 class KIOWidgetJobUiDelegateFactory : public KIO::JobUiDelegateFactory
